package ru.dbotthepony.kstarbound.defs.item

import com.google.gson.GsonBuilder
import ru.dbotthepony.kstarbound.io.KConcreteTypeAdapter
import ru.dbotthepony.kstarbound.registerTypeAdapter
import ru.dbotthepony.kvector.vector.ndouble.Vector2d
import ru.dbotthepony.kvector.vector.nint.Vector2i

data class ItemDefinition(
	val itemName: String,
	val price: Long = 0L,
	val rarity: ItemRarity = ItemRarity.COMMON,
	val category: String? = null,
	val inventoryIcon: String? = null,
	val description: String = "...",
	val shortdescription: String = "...",
	val itemTags: List<String> = listOf(),
	val learnBlueprintsOnPickup: List<String> = listOf(),
	val maxStack: Long = 9999L,
	val eventCategory: String? = null,
	val consumeOnPickup: Boolean = false,
	val pickupQuestTemplates: List<String> = listOf(),

	/**
	 * Используется в костях-ископаемых
	 */
	val race: String? = null,
	val displayImage: String? = null,
	val displayoffset: Vector2d? = null,

	/**
	 * Используется в костях-ископаемых
	 */
	val fossilSetName: String? = null,

	/**
	 * Используется в костях-ископаемых
	 */
	val setIndex: Int? = null,

	/**
	 * Используется в костях-ископаемых
	 */
	val setCount: Int? = null,

	/**
	 * Используется в костях-ископаемых
	 */
	val setCollectables: Map<String, String>? = null,

	/**
	 * Используется в костях-ископаемых
	 */
	val completeFossilIcon: String? = null,

	/**
	 * Используется в костях-ископаемых
	 */
	val completeFossilObject: String? = null,

	/**
	 * Используется в костях-ископаемых
	 */
	val completeSetDescriptions: FossilSetDescription? = null,

	/**
	 * Заставляет SAIL болтать
	 */
	val radioMessagesOnPickup: List<String> = listOf(),

	/**
	 * Топливо корабля
	 */
	val fuelAmount: Long? = null,
) {
	data class FossilSetDescription(
		val price: Long = 0L,
		val shortdescription: String = "...",
		val description: String = "..."
	)

	companion object {
		val ADAPTER = KConcreteTypeAdapter.Builder(ItemDefinition::class)
			.plain(ItemDefinition::itemName)
			.plain(ItemDefinition::price)
			.plain(ItemDefinition::rarity)
			.plain(ItemDefinition::category)
			.plain(ItemDefinition::inventoryIcon)
			.plain(ItemDefinition::description)
			.plain(ItemDefinition::shortdescription)

			.list(ItemDefinition::itemTags)
			.list(ItemDefinition::learnBlueprintsOnPickup)

			.plain(ItemDefinition::maxStack)
			.plain(ItemDefinition::eventCategory)
			.plain(ItemDefinition::consumeOnPickup)
			.list(ItemDefinition::pickupQuestTemplates)

			.plain(ItemDefinition::race)
			.plain(ItemDefinition::displayImage)
			.plain(ItemDefinition::displayoffset)
			.plain(ItemDefinition::fossilSetName)
			.plain(ItemDefinition::setIndex)
			.plain(ItemDefinition::setCount)
			.map(ItemDefinition::setCollectables, String::class)
			.plain(ItemDefinition::completeFossilIcon)
			.plain(ItemDefinition::completeFossilObject)

			.plain(ItemDefinition::completeSetDescriptions)
			.list(ItemDefinition::radioMessagesOnPickup)
			.plain(ItemDefinition::fuelAmount)

			.build()

		val FOSSIL_ADAPTER = KConcreteTypeAdapter.Builder(FossilSetDescription::class)
			.plain(FossilSetDescription::price)
			.plain(FossilSetDescription::shortdescription)
			.plain(FossilSetDescription::description)
			.build()

		fun registerGson(gsonBuilder: GsonBuilder) {
			gsonBuilder.registerTypeAdapter(ADAPTER)
			gsonBuilder.registerTypeAdapter(FOSSIL_ADAPTER)
		}
	}
}
